/**
 * Sirve de base para aprender los conceptos bsicos de utilizacin de 
 * clases en Java.
 * Permite la creacin de fichas situadas en un tablero,
 * el cual puede ser mostrado en la salida estndar.
 * Se permite hacer varios movimientos sobre las fichas en las 4 direcciones.
 * El tablero se puede mostrar con todas las fichas que se han creado,
 * o bien, mostrar slo el tablero de una ficha determinada
 * 
 * @author Javier Garca
 * 
 */
public class Ficha {
    /** Carcter que representar a la ficha en el tablero */
    public char simbolo;
    private int posicionX;
    private int posicionY;
    private int numMovimientos = 0;
	/** Tamao horizontal del tablero */
    public static final int TAM_X = 8;
	/** Tamao vertical del tablero */
    public static final int TAM_Y = 8;
    private static final char tablero[][] = new char[TAM_X][TAM_Y];

    /**
     * Crea una ficha en una posicin aleatoria, y se representa con
     * el carcter '#' e indica en la salida estndar la posicin en la
     * que ha sido creada
     */
    public Ficha() {
        posicionX = (int) (Math.random() * 8);
        posicionY = (int) (Math.random() * 8);
        simbolo = '#';
        tablero[posicionX][posicionY] = simbolo;
        System.out.println("La ficha se ha creado correctamente en " + dameCoordenadas());
    }

    /**
     * Crea una ficha en la posicin que se indique y representado por
     * el carcter indicado en el smbolo
     * @param simbolo Carcter que representar a la ficha en el tablero
     * @param posicionX Posicin en el eje X donde se colocar la ficha 
     * entre las posiciones 1 y 8
     * @param posicionY Posicin en el eje Y donde se colocar la ficha
     * entre las posiciones 1 y 8
     */
    public Ficha(char simbolo, int posicionX, int posicionY) {
        this.simbolo = simbolo;
        reiniciar(posicionX, posicionY);
    }

    /**
     * Sube la ficha una posicin. Si se supera el lmite del tablero,
     * se mover a la parte inferior.
     */
    public void subir() {
        mover(0, -1);
    }

    /**
     * Baja la ficha una posicin. Si se supera el lmite del tablero,
     * se mover a la parte superior.
     */
    public void bajar() {
        mover(0, 1);
    }

    /**
     * Mueve la ficha a la derecha una posicin. Si se supera el lmite del tablero,
     * se mover a la parte izquierda.
     */
    public void derecha() {
        mover(1, 0);
    }

    /**
     * Mueve la ficha a la izquierda una posicin. Si se supera el lmite del tablero,
     * se mover a la parte derecha.
     */
    public void izquierda() {
        mover(-1, 0);
    }

    private void mover(int incrementoX, int incrementoY) {
        int posXold = posicionX;
        int posYold = posicionY;
        tablero[posicionX][posicionY] = 0;
        posicionX = posicionX + incrementoX;
        posicionY = posicionY + incrementoY;
        comprobarPosicion();
        tablero[posicionX][posicionY] = simbolo;
        numMovimientos += Math.abs(posicionX - posXold) + Math.abs(posicionY - posYold);
    }

    private void comprobarPosicion() {
        if (posicionX < 0) {
            posicionX = 0;
        }
        if (posicionY < 0) {
            posicionY = 0;
        }
        if (posicionX >= TAM_X) {
            posicionX = TAM_X - 1;
        }
        if (posicionY >= TAM_Y) {
            posicionY = TAM_Y - 1;
        }
    }

    /**
     * Coloca la ficha en la posicin que se indique y se inicia de nuevo
     * la cuenta de movimientos
     * @param posicionX Posicin en el eje X donde se colocar la ficha
     * entre las posiciones 1 y 8
     * @param posicionY Posicin en el eje Y donde se colocar la ficha
     * entre las posiciones 1 y 8
     */
    public void reiniciar(int posicionX, int posicionY) {
        this.posicionX = posicionX - 1;
        this.posicionY = posicionY - 1;
        comprobarPosicion();
        tablero[this.posicionX][this.posicionY] = this.simbolo;
        this.numMovimientos = 0;
    }

    /**
     * Permite obtener las coordenadas de la posicin de la ficha
     * @return Devuelve una cadena de caracteres con las coordenadas de la
     * ficha entre parntesis y separadas por una coma
     */
    public String dameCoordenadas() {
        return "(" + (posicionX + 1) + "," + (posicionY + 1) + ")";
    }

    /**
     * Muestra en la salida estndar el tablero con todos las fichas que
     * se hayan creado.
     */
    public static void pintarTableroCompleto() {
        System.out.print(" ");
        for (int y = 0; y < TAM_Y; y++) {
            System.out.print("-");
        }
        System.out.println();
        for (int y = 0; y < TAM_Y; y++) {
            System.out.print("|");
            for (int x = 0; x < TAM_X; x++) {
                if (tablero[x][y] == 0) {
                    System.out.print(" ");
                } else {
                    System.out.print(tablero[x][y]);
                }
            }
            System.out.println("|");
        }
        System.out.print(" ");
        for (int y = 0; y < TAM_Y; y++) {
            System.out.print("-");
        }
        System.out.println();
    }

    /**
     * Muestra en la salida estndar el tablero, slo con la ficha sobre
     * la que se llama a este mtodo
     */
    public void pintarTablero() {
        System.out.print(" ");
        for (int y = 0; y < TAM_Y; y++) {
            System.out.print("-");
        }
        System.out.println();
        for (int y = 0; y < TAM_Y; y++) {
            System.out.print("|");
            for (int x = 0; x < TAM_X; x++) {
                if (x == posicionX && y == posicionY) {
                    System.out.print(simbolo);
                } else {
                    System.out.print(" ");
                }
            }
            System.out.println("|");
        }
        System.out.print(" ");
        for (int y = 0; y < TAM_Y; y++) {
            System.out.print("-");
        }
        System.out.println();
    }

    /**
     * Permite conocer si la ficha est en alguno de los bordes del tablero
     * @return true si est en alguno de los bordes, y false en caso contrario
     */
    public boolean estaEnBorde() {
        if (posicionX == 0 || posicionX == TAM_X - 1
                || posicionY == 0 || posicionY == TAM_Y - 1) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Permite conocer el nmero de movimientos que se han efectuado sobre
     * la ficha en el tablero
     * @return Nmero de movimientos efectuados en la ficha sobre el tablero
     */
    public int dameNumMovimientos() {
        return numMovimientos;
    }
}